// TODO make imports dynamic/only load what is needed for the method below
import * as Block from "multiformats/block";
import { CID } from "multiformats/cid";
import * as dagPb from "@ipld/dag-pb";
import * as dagCbor from "@ipld/dag-cbor";
import * as raw from "multiformats/codecs/raw";
import { sha256 as hasher } from "multiformats/hashes/sha2";
import { base58btc } from "multiformats/bases/base58";
const mhtype = "sha2-256";
const defaultBase = base58btc;
const unsupportedCodecError = () => new Error("unsupported codec");
const cidifyString = (str) => {
    if (!str) {
        return str;
    }
    if (Array.isArray(str)) {
        return str.map(cidifyString);
    }
    return CID.parse(str);
};
const stringifyCid = (cid, options = {}) => {
    if (!cid || typeof cid === "string") {
        return cid;
    }
    if (Array.isArray(cid)) {
        return cid.map(stringifyCid);
    }
    const base = options.base || defaultBase;
    if (cid["/"]) {
        return cid["/"].toString(base);
    }
    return cid.toString(base);
};
const codecCodes = {
    [dagPb.code]: dagPb,
    [dagCbor.code]: dagCbor,
    [raw.code]: raw,
};
const codecMap = {
    raw: raw,
    "dag-pb": dagPb,
    "dag-cbor": dagCbor,
};
async function read(ipfs, cid, options = {}) {
    cid = cidifyString(stringifyCid(cid));
    const codec = codecCodes[cid.code];
    const bytes = await ipfs.block.get(cid, { timeout: options.timeout });
    const block = await Block.decode({ bytes, codec, hasher });
    if (block.cid.code === dagPb.code) {
        return JSON.parse(new TextDecoder().decode(block.value.Data));
    }
    if (block.cid.code === dagCbor.code) {
        const value = block.value;
        const links = options.links || [];
        links.forEach((prop) => {
            if (value[prop]) {
                value[prop] = stringifyCid(value[prop], options);
            }
        });
        return value;
    }
    if (block.cid.code === raw.code) {
        return block.value;
    }
    throw new Error("Unsupported");
}
async function write(ipfs, format, value, options = {}) {
    const codec = codecMap[format];
    if (!codec)
        throw unsupportedCodecError();
    if (codec.code === dagPb.code) {
        value = typeof value === "string" ? value : JSON.stringify(value);
        value = { Data: new TextEncoder().encode(value), Links: [] };
    }
    if (codec.code === dagCbor.code) {
        const links = options.links || [];
        links.forEach((prop) => {
            if (value[prop]) {
                value[prop] = cidifyString(value[prop]);
            }
        });
    }
    const block = await Block.encode({ value, codec, hasher });
    await ipfs.block.put(block.bytes, {
        /* cid: block.cid.bytes, */
        version: block.cid.version,
        format,
        mhtype,
        pin: options.pin,
        timeout: options.timeout,
    });
    const cid = codec.code === dagPb.code ? block.cid.toV0() : block.cid;
    const cidString = cid.toString(options.base || defaultBase);
    return cidString;
}
async function rm(ipfs, hash) {
    try {
        await ipfs.pin.rm(hash);
    }
    catch (error) {
        // not pinned // TODO add bettor error handling
    }
    for await (const result of ipfs.block.rm((hash instanceof CID ? hash : CID.parse(hash)))) {
        if (result.error) {
            throw new Error(`Failed to remove block ${result.cid} due to ${result.error.message}`);
        }
    }
}
export default {
    read,
    write,
    rm,
};
//# sourceMappingURL=index.js.map